import java.io.File;
import java.util.ArrayList;


public class Game {
	private static String version = "1.1";
	public static boolean isRunning = false;
	public static GameMode mode;
	public static final java.util.Random r = new java.util.Random();
	private static boolean debug;
	private static ArrayList<Snake> snakes = new ArrayList<Snake>();
	private static Console c;/* = new Console();*/
	private static Window w;
	private static GamePanel gp;
	public static int score = 0;
	public static double prize = 100;
	public static int speed = 0;
	private static String path = System.getenv("APPDATA") + "/.snake";
	private static long last;
	private long duration;
	
	public static void main(String[] args){
		new Game().run(args);
	}
	public void run(String[] args) {
		new File(path).mkdirs();
		gp = new GamePanel();
		w = new Window();
		for(int x = 0; x < 64; x++) {
			for(int y = 0; y < 64; y++) {
				if(x == 0 || y == 0 || x == 63 || y == 63) { //check for border tile
					new Obstacle(x, y);
				}
			}
		}
		new MainMenu().show();
		last = System.nanoTime();
		while(true) {
			if(Menu.getCurrentMenu() == null || Menu.getCurrentMenu() instanceof GameOverMenu || Menu.getCurrentMenu() instanceof ScoreBoard) {
				for(Snake s: snakes)
					s.move();			
			}
			if(Menu.getCurrentMenu() != null)
				Menu.getCurrentMenu().update();
			String title = "Snake version " + version;
			if(mode != GameMode.SURVIVAL) {
				if(snakes.size() == 1)
					title += " - Score: " + snakes.get(0).getScore();
				else
					for(Snake s: snakes) {
						title += " - " + s.getName() + ": " + s.getScore();
					}
			}
			w.setTitle(title + (mode == GameMode.SINGLEPLAYER || mode == GameMode.FOODHUNT? " +" + (int) prize: " by Raptilicus"));
			if(prize > 0 && Menu.getCurrentMenu() instanceof ScoreBoard)
				prize -= 0.02;
			gp.repaint();
			duration = (System.nanoTime() - last);
			try {
				if(duration < 6000000) {
					int millis = 0;
					long nanos = duration;
					while(nanos >= 1000000) {
						millis++;
						nanos -= 1000000;
					}
//					System.out.println(-((6 - millis) * 1000000 + nanos) + duration);
					Thread.sleep(6 - millis, (int) nanos);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
			last = System.nanoTime();
		}
		
	}
	
	public static void start(GameMode gm) {
		stop();
		switch(gm) {
		case SINGLEPLAYER: snakes.add(new Snake("Player", 32, 32, 0)); break;
		case FOODHUNT:
		case SURVIVAL: snakes.add(new Snake("Player 1", 16, 48, 0)); snakes.add(new Snake("Player 2", 48, 16, 2)); break;
		case NONE: mode = gm; return;
		}
		isRunning = true;
		mode = gm;
		Food.newFood();
		prize = 100;
		new ScoreBoard().show();
		System.out.println("Starting game with game mode " + gm);
	}
	@SuppressWarnings("unchecked")
	public static void stop() {
		for(Snake s: snakes) {
			s.delete();
		}
		snakes.clear();
		for(Tile t: (ArrayList<Tile>) Tile.tiles.clone())
			if(t instanceof Tail)
				t.delete();
		if(Food.f != null) {
			Food.f.delete();
			Food.f = null;
		}
		isRunning = false;
		mode = GameMode.NONE;
	}
	
	public static String getVersion() {return version;}
	public static boolean isDebugging() {return debug;}
	public static void setDebugging(boolean debug) {Game.debug = debug;}
	public static Console getConsole() {return c;}
	public static Window getWindow() {return w;}
	public static GamePanel getGamePanel() {return gp;}
	public static void addPoints() {score += prize; prize = 100;}
	public static String getGameFolder() {return path;}
	public static ArrayList<Snake> getSnakes() {return snakes;}
	
	public static void increaseSpeed() {
		if(speed < 150)
			speed += 5;
	}
	
	public static void exit() {
		if(Menu.getCurrentMenu() instanceof MainMenu || true) {
			System.out.println("Exiting ...");
			System.exit(0);
		}
	}
}

enum GameMode {
	NONE,
	SINGLEPLAYER,
	FOODHUNT,
	SURVIVAL;
}
