import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;


public class GamePanel extends javax.swing.JPanel implements Runnable {
	private static final long serialVersionUID = 1552746400473185110L;
	InputManager im = new InputManager();
	int bw = getBoxWidth(), sw = getSpanWidth(), bx = getBorderX(), by = getBorderY();

	public GamePanel() {
		setBackground(Color.black);
		setFocusable(true);
		requestFocus();
		addKeyListener(im);
		addMouseListener(im);
		addMouseMotionListener(im);
//		new Thread(this).start();
	}
	
	public void run() {
		while(true) {
			repaint();
			try {
				Thread.sleep(5);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
	
	@SuppressWarnings("unchecked")
	public void paint(Graphics g) {
		super.paint(g);
		g.setColor(new Color(30, 30, 30));
		bw = getBoxWidth();
		sw = getSpanWidth();
		bx = getBorderX();
		by = getBorderY();
		for(int x = 0; x < 64; x++) {
			for(int y = 0; y < 64; y++) {
				g.fillRect(bx + x * (bw + sw), by + y * (bw + sw), bw, bw);
			}
		}
		Menu m = Menu.getCurrentMenu();
		if(m instanceof ScoreBoard) {
			for(Label l: m.getLabels()) {
				for(Tile t: l.getTiles()) {
					draw(t, g);
				}
			}
		}
		for(Tile t: Tile.tiles) {
			if(!(t instanceof Tail))
				draw(t, g);
		}
		for(Snake s: (ArrayList<Snake>) Game.getSnakes().clone()) {
			for(Tail t: (ArrayList<Tail>) s.getTail().clone()) {
				draw(t, g);
			}
		}
		if(m != null && !(m instanceof ScoreBoard)) {
			for(Label l: m.getLabels()) {
				for(Tile t: l.getTiles()) {
					draw(t, g);
				}
			}
		}
		g.dispose();
		updateUI();
	}
	
	private void draw(Tile t, Graphics g) {
		g.setColor(t.getColor());
		int x = t.getX(), y = t.getY();
		g.fillRect(bx + x * (bw + sw), by + y * (bw + sw), bw, bw);
	}
	
	private int getBoxWidth() {
		if(getWidth() < getHeight()) {
			return getWidth() / 64 - getSpanWidth();
		} else {
			return getHeight() / 64 - getSpanWidth();
		}
	}
	
	private int getSpanWidth() {
	if(getWidth() < getHeight()) {
		return getWidth() / 640;
	} else {
		return getHeight() / 640;
	}
	}
	
	private int getBorderX() {
		return (getWidth() - (getBoxWidth() + getSpanWidth()) * 64) / 2; 
	}
	
	private int getBorderY() {
		return (getHeight() - (getBoxWidth() + getSpanWidth()) * 64) / 2; 
	}
	
}
