import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.HashMap;


public class Label {
	protected int x, y, w, h;
	protected Color[][] map;
	protected ArrayList<FontTile> tiles = new ArrayList<FontTile>();
	protected static HashMap<String, Color[][]> maps = new HashMap<String, Color[][]>();
	protected static HashMap<String, Label> labels = new HashMap<String, Label>();

	public Label(String name, int x, int y) {
		this.x = x;
		this.y = y;
		map = getMap(name);
		if(!(this instanceof Button)) {
			create();
		}
		if(!labels.containsKey(name))
			labels.put(name, this);
	}
	
	public Label create() {
		Color[][] c = map;
		tiles.clear();
		for(int x = 0; x < w; x++) {
			for(int y = 0; y < h; y++) {
				if(c[x][y] != null) {
					FontTile f = new FontTile(x + this.x, y + this.y);
					f.setColor(c[x][y]);
					tiles.add(f);
				}
			}
		}
		return this;
	}
	
	public void delete() {
		for(FontTile t: tiles) {
			t.delete();
		}
		tiles.clear();
	}
	
	protected Color[][] getMap(String name) {
		if(maps.containsKey(name)) return maps.get(name);
		Image i = new javax.swing.ImageIcon(this.getClass().getResource(name + ".png")).getImage();
		BufferedImage bi = new BufferedImage(i.getWidth(null), i.getHeight(null), BufferedImage.TYPE_INT_ARGB);
		Graphics g = bi.getGraphics();
		g.drawImage(i, 0, 0, null);
		g.dispose();
		return getMap(bi);
	}
	
	protected Color[][] getMap(BufferedImage i) {
		Color[][] map = new Color[w = i.getWidth(null)][h = i.getHeight(null)];
		for(int x = 0; x < w; x++) {
			for(int y = 0; y < h; y++) {
				map[x][y] = new Color(i.getRGB(x, y), true);
			}
		}
		return map;
	}
	public ArrayList<FontTile> getTiles() {return tiles;}
}
