import java.util.ArrayList;


public abstract class Menu {
	protected ArrayList<Button> buttons = new ArrayList<Button>();
	protected ArrayList<Label> labels = new ArrayList<Label>();
	private static Menu currentMenu;
	
	public void show() {
		currentMenu = this;
		System.out.println(getClass().getName() + " opened");
	}
	public void hide() {
		currentMenu = null;
	}
	
	protected void addButton(Button b) {
		buttons.add(b);
		b.mouseOver = InputManager.isMouseOverButton(b);
	}
	
	protected void addLabel(Label l) {
		labels.add(l);
	}
	
	public abstract void onClose();
	public void update() {}
	
	public ArrayList<Label> getLabels() {
		ArrayList<Label> l = new ArrayList<Label>();
		l.addAll(labels);
		l.addAll(buttons);
		return l;
	}
	public ArrayList<Button> getButtons() {return buttons;}
	
	public static Menu getCurrentMenu() {
		return currentMenu;
	}
}

class MainMenu extends Menu {
	static Label snake = new Label("Snake", 0, 0);
	Button P1 = new Button("1P", 14, 27) {public void onClick() {hide(); Game.start(GameMode.SINGLEPLAYER);}};
	Button P2 = new Button("2P", 14, 37) {public void onClick() {hide(); new MultiplayerMenu().show();}};
	Button exit = new Button("Exit", 14, 47) {public void onClick() {Game.exit();}};
	
	
	public MainMenu() {
		addLabel(snake);
		addButton(P1);
		addButton(P2);
		addButton(exit);
	}

	public void onClose() {Game.exit();}
}

class PauseMenu extends MainMenu {
	Button mainMenu = new Button("MainMenu", 14, 47) {public void onClick() {Game.stop(); hide(); new MainMenu().show();}};
	
	public PauseMenu() {
		addLabel(snake);
		switch(Game.getSnakes().size()) {
		case 1: 
			addButton(new Button("Continue", 14, 27) {public void onClick() {onClose();}});
			buttons.remove(P1); break;
		case 2:
			addButton(new Button("Continue", 14, 37) {public void onClick() {onClose();}});
			buttons.remove(P2); break;
		}
		buttons.remove(super.exit);
		addButton(mainMenu);
	}
	
	public void onClose() {hide(); new ScoreBoard().show();}
	
}

class GameOverMenu extends Menu {
	static Label go = new Label("GameOver", 0, 0);
	static Label winner = new Label("Winner", 0, 0);
	static Label scores = new Label("Scores", 0, 0);
	static Label one = new Label("1", 34, 5);
	static Label two = new Label("2", 34, 5);
	Button retry = new Button("Retry", 14, 37) {public void onClick() {Game.start(Game.mode);}};
	Button mainmenu = new Button("MainMenu", 14, 47) {public void onClick() {Game.stop(); new MainMenu().show();}};
	
	public GameOverMenu(Snake triggerer) {
		if(Game.mode == GameMode.SINGLEPLAYER)
			addLabel(go);
		else {
			addLabel(winner);
			if(Game.mode == GameMode.SURVIVAL) {
				if(Game.getSnakes().get(0) == triggerer) {
					addLabel(two);
				} else {
					addLabel(one);
				}
			} else if(Game.mode == GameMode.FOODHUNT) {
				if(Game.getSnakes().get(0).getScore() == Game.getSnakes().get(1).getScore()) {
					if(Game.getSnakes().get(0) == triggerer) {
						addLabel(one);
					} else {
						addLabel(two);
					}
				} else if(Game.getSnakes().get(0).getScore() > Game.getSnakes().get(1).getScore()) {
					addLabel(one);
				} else {
					addLabel(two);
				}
			}
		}
		if(Game.mode == GameMode.FOODHUNT) {
//			addLabel(scores);
		}
		if(Game.mode != GameMode.SURVIVAL) {
			int snake = 0;
			int offx = 44;
			if(Game.mode == GameMode.SINGLEPLAYER) {
				int number = 0;
				Snake s = Game.getSnakes().get(0);
				char[] ca = ("" + s.getScore()).toCharArray();
				addLabel(new Label("Score", 21 - ca.length * 2, 13));
				for(char c: ca) {
					addLabel(new Label("" + c, offx + number * 4 - ca.length * 2, 13));
					number++;
				}
			} else {
				offx = 49;
				int off = ("" + (Game.getSnakes().get(0).getScore() > Game.getSnakes().get(1).getScore()? Game.getSnakes().get(0): Game.getSnakes().get(1)).getScore()).length();
				addLabel(new Label("Player", 16 - off * 2, 13));
				addLabel(new Label("1", 41 - off * 2, 13));
				addLabel(new Label("Player", 16 - off * 2, 20));
				addLabel(new Label("2", 41 - off * 2, 20));
				for(Snake s: Game.getSnakes()) {
					int number = 0;
					for(char c: ("" + s.getScore()).toCharArray()) {
						addLabel(new Label("" + c, offx + number * 4 - off * 2, 13 + snake * 7));
						number++;
					}
					snake++;
				}
			}
		}
		addButton(retry);
		addButton(mainmenu);
	}
	
	public void onClose() {
		mainmenu.onClick();
	}
}

class ScoreBoard extends Menu {
	Label score = new Label("ScoreBG", 5, 5);
	ArrayList<Label> numbers = new ArrayList<Label>();
	
	public ScoreBoard() {
		if(Game.mode == GameMode.SURVIVAL) return;
		if(Game.mode == GameMode.SINGLEPLAYER) {
			addLabel(score);
		} else {
			addLabel(new Label("PlayerBG", 5, 5));
			addLabel(new Label("1BG", 30, 5));
			addLabel(new Label("PlayerBG", 5, 12));
			addLabel(new Label("2BG", 30, 12));
		}
	}
	
	public void update() {
		if(Game.mode == GameMode.SURVIVAL) return;
		labels.removeAll(numbers);
		numbers.clear();
		int longestScore = 0;
		for(Snake s: Game.getSnakes())
			if(s.getScore() > longestScore)
				longestScore = s.getScore();
		longestScore = ("" + longestScore).length();
		for(int i = 0; i < Game.getSnakes().size(); i++) {
			Snake s = Game.getSnakes().get(i);
			for(char c: ("" + s.getScore()).toCharArray()) {
				addLabel(new Label(c + "BG", 50 + longestScore * 4, 5 + i * 7));
			}
		}
		labels.addAll(numbers);
	}
	
	public void onClose() {
		new PauseMenu().show();
	}
}

class MultiplayerMenu extends Menu {
	Button foodHunt = new Button("FoodHunt", 14, 27) {public void onClick() {Game.start(GameMode.FOODHUNT);}};
	Button survival = new Button("Survival", 14, 37) {public void onClick() {Game.start(GameMode.SURVIVAL);}};
	Button mainMenu = new Button("Back", 14, 47) {public void onClick() {new MainMenu().show();}};
	Button back = new Button("Back", 14, 47) {public void onClick() {new PauseMenu().show();}};
	
	public MultiplayerMenu() {
		addLabel(MainMenu.snake);
		addButton(foodHunt);
		addButton(survival);
		if(Game.isRunning)
			addButton(back);
		else
			addButton(mainMenu);
	}
	
	public void onClose() {
		if(Game.isRunning)
			back.onClick();
		else
			mainMenu.onClick();
	}
	
}